from django.db import models

# Create your models here.

# *** 银行接口模型 ***
from django_extensions.db.models import TimeStampedModel


class InterfaceHistory(TimeStampedModel):
    """接口日志。

    记录每一次接口的交互日志。方便未来出现争议、或排查问题时使用
    """

    class Meta:
        verbose_name = verbose_name_plural = '接口日志'

    def __str__(self):
        return str(self.id)

    service_name = models.CharField(verbose_name='服务', blank=True, null=True, max_length=500)
    url = models.CharField(verbose_name='URL', max_length=500, blank=True, null=True)
    method = models.CharField(verbose_name='方式', blank=True, null=True, max_length=50)
    payload = models.TextField(verbose_name='输入参数', blank=True, null=True)

    http_status = models.CharField(verbose_name='HTTP状态码', blank=True, null=True, max_length=50)
    http_response_header = models.TextField(verbose_name='HTTP响应头信息', blank=True, null=True)
    http_response_content = models.TextField(verbose_name='HTTP响应消息体', blank=True, null=True)
    http_error = models.TextField(verbose_name='异常信息', blank=True, null=True)
    time_it = models.IntegerField(verbose_name='响应时间', default=0, help_text='毫秒')

    @classmethod
    def log(cls, url=None, service_name=None, method=None, payload=None, http_status=None,
            http_response_header=None, http_response_content=None, http_error=None, time_it=0):
        # todo 未来这里应该异步记录日志
        # todo 未来要考虑是否有与安全有关的信息应该去掉
        obj = cls.objects.create(
            url=url,
            service_name=service_name,
            method=method,
            payload=payload,
            http_status=http_status,
            http_response_header=http_response_header,
            http_response_content=http_response_content,
            http_error=http_error,
            time_it=time_it,
        )
        return obj



# coding: utf-8
#
# import django.dispatch
# from django.dispatch import receiver
#
# httplog_signal = django.dispatch.Signal(
#     providing_args=['server_addr', 'service_name', 'method', 'payload', 'http_status',
#                     'http_response_header', 'http_response_content', 'http_error', 'time_it'])
#
#
# @receiver(httplog_signal, dispatch_uid="httplog.httplog_receiver")
# def httplog_signal_handler(sender, **kwargs):
#     print('my_signal received')
#     print(kwargs)



# Generated by Django 2.1.11 on 2019-08-27 14:01

from django.db import migrations, models
import django_extensions.db.fields


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='InterfaceHistory',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created', django_extensions.db.fields.CreationDateTimeField(auto_now_add=True, verbose_name='created')),
                ('modified', django_extensions.db.fields.ModificationDateTimeField(auto_now=True, verbose_name='modified')),
                ('service_name', models.CharField(blank=True, max_length=500, null=True, verbose_name='服务名称')),
                ('url', models.CharField(blank=True, max_length=500, null=True, verbose_name='服务地址')),
                ('method', models.CharField(blank=True, max_length=50, null=True, verbose_name='方式')),
                ('payload', models.TextField(blank=True, null=True, verbose_name='输入参数')),
                ('http_status', models.CharField(blank=True, max_length=50, null=True, verbose_name='HTTP请求状态码')),
                ('http_response_header', models.TextField(blank=True, null=True, verbose_name='HTTP响应头信息')),
                ('http_response_content', models.TextField(blank=True, null=True, verbose_name='HTTP响应消息体')),
                ('http_error', models.TextField(blank=True, null=True, verbose_name='HTTP连接错误信息')),
                ('time_it', models.IntegerField(default=0, help_text='毫秒', verbose_name='服务响应时间')),
            ],
            options={
                'verbose_name': '接口日志',
                'verbose_name_plural': '接口日志',
            },
        ),
    ]



# coding: utf-8
from django.http import HttpRequest

from weblogtools.models import InterfaceHistory
import requests
import json
import time

MAX_KEEPING_DATA_LENGTH = 1000


def http_log(server_addr=None, service_name=None, method=None, payload=None, http_status=None,
             http_response_header=None, http_response_content=None, http_error=None, time_it=0):
    """记录http日志"""
    InterfaceHistory.log(
        server_addr, service_name, method, payload, http_status,
        http_response_header, http_response_content, http_error, time_it
    )


def http_log_from_response(service_name, response: requests.Response, timeit=0):
    """调别人通过这里记录日志"""
    request = response.request
    # request = requests.Request()
    body = request.body[:MAX_KEEPING_DATA_LENGTH].decode(errors='ignore')
    http_log(
        request.url, service_name, request.method, body, response.status_code, json.dumps(dict(response.headers)),
        response.content[:MAX_KEEPING_DATA_LENGTH].decode(errors='ignore'), None,
        timeit
    )


def http_log_from_request(service_name, request: HttpRequest, status_code, response_headers,
                          response_content, timeit=0):
    """回调的情况调这里记录日志"""
    body = request.body[:MAX_KEEPING_DATA_LENGTH].decode(errors='ignore')
    http_log(
        request.path, service_name, request.method, body, status_code, json.dumps(dict(response_headers)),
        response_content[:MAX_KEEPING_DATA_LENGTH],
        None, timeit
    )


class TimeIt(object):
    """计时器。

    用法::

        with httplog.TimeIt() as timeit:
            res = requests.post(url, json=payload)
        httplog.http_log_from_response('upload_case', res, timeit.duration)

    """
    def __init__(self):
        self.start = 0
        self.end = 0

    def __enter__(self):
        self.start = time.time()
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        self.end = time.time()

    @property
    def duration(self):
        return int((self.end - self.start) * 1000)



from django.contrib import admin

from . import models as m


# Register your models here.

@admin.register(m.InterfaceHistory)
class InterfaceHistoryAdmin(admin.ModelAdmin):
    change_form_template = 'weblogtools/change_form.html'
    list_display = (
        'id', 'created', 'service_name', 'time_it', 'http_status', 'http_response_content',)
    date_hierarchy = 'created'
    list_filter = ('service_name', 'http_status')
    fields = (
        ('service_name', 'url'),
        ('http_status', 'time_it', 'created'),
        ('payload',),
        ('http_response_header', 'http_response_content'),
        ('http_error',)
    )
    ordering = ('-created',)
    search_fields = ('service_name', 'http_response_content', 'input_common', 'input_service')
    list_display_links = ('id', 'http_response_content')

    def has_delete_permission(self, request, obj=None):
        return True

    def has_add_permission(self, request):
        return False

    def has_change_permission(self, request, obj=None):
        return False



import os
from setuptools import find_packages, setup

with open(os.path.join(os.path.dirname(__file__), 'README.rst')) as readme:
    README = readme.read()

# allow setup.py to be run from any path
os.chdir(os.path.normpath(os.path.join(os.path.abspath(__file__), os.pardir)))

setup(
    name='django-weblogtools',
    version='0.1.2',
    packages=find_packages(),
    include_package_data=True,
    license='BSD License',  # example license
    description='A simple Django app to log http request / response with external services.',
    long_description=README,
    url='https://github.com/daimon99/django-httplog',
    author='Jian Dai',
    author_email='jian.dai@gmail.com',
    classifiers=[
        'Environment :: Web Environment',
        'Framework :: Django',
        'Framework :: Django :: 2.0',  # replace "X.Y" as appropriate
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',  # example license
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Topic :: Internet :: WWW/HTTP',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
    ],
)
